Package: NormalyzerDE
Title: Evaluation of normalization methods and calculation of
        differential expression analysis statistics
Version: 1.30.0
Author: Jakob Willforss
Authors@R: c(
    person("Jakob", "Willforss", email="jakob.willforss@hotmail.com", role=c("aut", "cre")),
    person("Aakash", "Chawade", role="aut"),
    person("Fredrik", "Levander", email="fredrik.levander@immun.lth.se", role=c("aut", "ths")),
    person("Måns", "Zamore", email="mans.bioc@zamore.se", role=c("aut")))
Description: NormalyzerDE provides screening of normalization methods for 
    LC-MS based expression data. It calculates a range of normalized matrices 
    using both existing approaches and a novel time-segmented approach, 
    calculates performance measures and generates an evaluation report. 
    Furthermore, it provides an easy utility for Limma- or ANOVA- based 
    differential expression analysis.
Imports: vsn, preprocessCore, limma, MASS, ape, car, ggplot2, methods,
        utils, stats, SummarizedExperiment, matrixStats, ggforce
Suggests: knitr, testthat, rmarkdown, roxygen2, hexbin, BiocStyle
VignetteBuilder: knitr
biocViews: Normalization, MultipleComparison, Visualization, Bayesian,
        Proteomics, Metabolomics, DifferentialExpression
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://computationalproteomics.github.io/NormalyzerDE/,
        https://github.com/ComputationalProteomics/NormalyzerDE
Depends: R (>= 4.1.0)
git_url: https://git.bioconductor.org/packages/NormalyzerDE
git_branch: RELEASE_3_23
git_last_commit: a701c99
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:37:52 UTC; biocbuild
Maintainer: Jakob Willforss <jakob.willforss@hotmail.com>
