# Generated by roxygen2: do not edit by hand

export(NormalyzerEvaluationResults)
export(NormalyzerResults)
export(NormalyzerStatistics)
export(analyzeNormalizations)
export(calculateContrasts)
export(generateAnnotatedMatrix)
export(generatePlots)
export(generateStatsReport)
export(getRTNormalizedMatrix)
export(getSmoothedRTNormalizedMatrix)
export(getVerifiedNormalyzerObject)
export(globalIntensityNormalization)
export(meanNormalization)
export(medianNormalization)
export(normMethods)
export(normalyzer)
export(normalyzerDE)
export(performCyclicLoessNormalization)
export(performGlobalRLRNormalization)
export(performQuantileNormalization)
export(performSMADNormalization)
export(performVSNNormalization)
export(reduceTechnicalReplicates)
export(setupJobDir)
export(setupRawContrastObject)
export(setupRawDataObject)
export(setupTestData)
export(writeNormalizedDatasets)
import(MASS)
import(limma)
import(methods)
import(preprocessCore)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
