% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performGlobalRLRNormalization}
\alias{performGlobalRLRNormalization}
\title{Global linear regression normalization}
\usage{
performGlobalRLRNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized matrix
}
\description{
Log2 transformed data is normalized by robust linear regression using
the function "rlm" from the MASS package.
}
\examples{
data(example_data_only_values_small)
normMatrix <- performGlobalRLRNormalization(example_data_only_values)
}
