% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotReplicateVariance}
\alias{plotReplicateVariance}
\title{Generate normalization replicate variance summary by displaying
CV (coefficient of variance), MAD (mean of intragroup median absolute
deviation) and PEV (Pooled Estimate of Variance) as mean of intragroups}
\usage{
plotReplicateVariance(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Generate normalization replicate variance summary by displaying
CV (coefficient of variance), MAD (mean of intragroup median absolute
deviation) and PEV (Pooled Estimate of Variance) as mean of intragroups
}
\keyword{internal}
